@@ -563,10 +563,11 @@ dvdnav_status_t dvdnav_get_position(dvdnav_t *this, uint32_t *pos,
     *len += cell->last_sector - cell->first_sector + 1;
   }
 
-  assert((signed)*pos != -1);
-
   pthread_mutex_unlock(&this->vm_lock);
 
+  if((signed)*pos == -1)
+    return DVDNAV_STATUS_ERR;
+
   return DVDNAV_STATUS_OK;
 }
 
@@ -586,6 +587,11 @@ dvdnav_status_t dvdnav_get_position_in_title(dvdnav_t *this,
     return DVDNAV_STATUS_ERR;
   }
 
+  if (state->pgc->program_map == NULL) {
+    printerr("Program map missing.");
+    return DVDNAV_STATUS_ERR;
+  }
+
   /* Get current sector */
   cur_sector = this->vobu.vobu_start + this->vobu.blockN;
 
@@ -638,6 +644,11 @@ uint32_t dvdnav_describe_title_chapters(dvdnav_t *this, int32_t title, uint64_t
   if(!tmp)
     goto fail;
 
+  if(!ptt) {
+      printerr("ptt NULL");
+      goto fail;
+  }
+
   length = 0;
   for(i=0; i<parts; i++) {
     uint32_t cellnr, endcellnr;
